use specs::World;

use crate::map_builders::bsp_dungeon::BspDungeonBuilder;
use crate::map_builders::bsp_interior::BspInteriorBuilder;
use crate::map_builders::simple_map::SimpleMapBuilder;
use crate::{Map, Position};
use crate::map_builders::cellular_automata::CellularAutomataBuilder;
use crate::map_builders::drunkard::{DrunkardSettings, DrunkardsWalkBuilder, DrunkSpawnMode};

mod bsp;
mod bsp_dungeon;
mod bsp_interior;
mod cellular_automata;
mod common;
mod simple_map;
mod drunkard;

pub trait MapBuilder {
    fn build_map(&mut self);
    fn spawn_entities(&mut self, ecs: &mut World);
    fn get_map(&self) -> Map;
    fn get_starting_position(&self) -> Position;
    fn get_snapshot_history(&self) -> Vec<Map>;
    fn take_snapshot(&mut self);
}

pub fn new_random_builder(new_depth: i32) -> Box<dyn MapBuilder> {
    let mut rng = rltk::RandomNumberGenerator::new();
    //let builder_choice = rng.roll_dice(1, 7);
    let builder_choice = 4;
    match builder_choice {
        6 => Box::new(DrunkardsWalkBuilder::open_area(new_depth)),
        5 => Box::new(DrunkardsWalkBuilder::open_halls(new_depth)),
        4 => Box::new(DrunkardsWalkBuilder::winding_passages(new_depth)),
        3 => Box::new(CellularAutomataBuilder::new(new_depth)),
        2 => Box::new(BspInteriorBuilder::new(new_depth)),
        1 => Box::new(SimpleMapBuilder::new(new_depth)),
        _ => Box::new(BspDungeonBuilder::new(new_depth)),
    }
}
